'''
.. module:: stats
   :platform: Linux
   :synopsis: Wrapper package module containing the VAR and FAVAR modules each of which contain their respective classes from which
              empirical macro models can be instantiated with data.

.. moduleauthor:: Eric M. Scheffel <eric.scheffel@nottingham.edu.cn>


'''