'''
.. module:: parsers
   :platform: Linux
   :synopsis: Wrapper module for a number of parsing modules which are crucial for the functioning of PyMacLab. The first parser
              collects raw data (i.e. lines) from the DSGE model files, while the second parser carefully extracts useful information
              and then stores this by attaching it to the DSGE model instance.

.. moduleauthor:: Eric M. Scheffel <eric.scheffel@nottingham.edu.cn>


'''